pub mod svg;
use svg::*;

pub mod key;
use key::*;

use askama::Template;
use derive_more::From;
use std::{fs::OpenOptions, io::Write};
use lazy_static::lazy_static;

#[derive(Template)]
#[template(path = "document.xml")]
struct DocumentTemplate<'a> {
    dimensions: &'a DocumentDimensions,
    box_size: SVGMeasure,
    positions: Vec<(&'a Key<'a>, (SVGMeasure, SVGMeasure))>,
}

struct DocumentDimensions {
    width: SVGMeasure,
    height: SVGMeasure,
}

#[derive(From, Debug)]
enum Error {
    Io(std::io::Error),
    Template(askama::Error),
}

fn positions<'a>(
    keys: &'a Vec<Key>,
    size: SVGMeasure,
    margin: SVGMeasure,
    padding: SVGMeasure,
    dimensions: &DocumentDimensions,
) -> Vec<(&'a Key<'a>, (SVGMeasure, SVGMeasure))> {
    let grid = size + padding;
    let mut x = margin;
    let mut y = margin;
    let mut positions = Vec::with_capacity(keys.len());
    for key in keys {
        if let Key::Break = key {
            x = margin;
            y = y + grid;
            continue;
        }
        if x + grid > dimensions.width - margin {
            x = margin;
            y = y + grid;
        }
        positions.push((key, (x, y)));
        x = x + size * key.width_mod() + padding;
    }
    positions
}

fn main() -> Result<(), Error> {
    let mut file = OpenOptions::new()
        .write(true)
        .truncate(true)
        .create(true)
        .open("output.svg")?;
    let dimensions = DocumentDimensions {
        width: SVGMeasure::new(8.5, SVGUnit::Inch),
        height: SVGMeasure::new(11.0, SVGUnit::Inch),
    };
    let box_size = SVGMeasure::new(14.0, SVGUnit::Millimeter);
    let padding = SVGMeasure::new(2.5, SVGUnit::Millimeter);
    let margin = SVGMeasure::new(0.25, SVGUnit::Inch);
    let document = DocumentTemplate {
        positions: positions(&KEYS, box_size, margin, padding, &dimensions),
        dimensions: &dimensions,
        box_size,
    };
    write!(file, "{}", document.render()?)?;
    Ok(())
}

lazy_static! {
    static ref KEYS: Vec<Key<'static>> = {
        let mut keys = OYAYUBI.to_vec();
        keys.push(Key::Break);
        keys.push(single("親指左", 3.0));
        keys.push(single("親指右", 3.0));
        keys.push(Key::Break);
        keys.push(icon("assets/nixos.svg", 1.0));
        keys.push(icon("assets/playpause.svg", 1.0));
        keys.push(icon("assets/previous.svg", 1.0));
        keys.push(icon("assets/next.svg", 1.0));
        keys
    };
}